# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, TYPE_CHECKING

from azure.core.rest import HttpRequest, HttpResponse
from azure.mgmt.core import ARMPipelineClient

from . import models as _models
from ._configuration import AlertsManagementClientConfiguration
from ._serialization import Deserializer, Serializer
from .operations import (
    AlertProcessingRulesOperations,
    AlertsOperations,
    Operations,
    PrometheusRuleGroupsOperations,
    SmartGroupsOperations,
)

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from azure.core.credentials import TokenCredential


class AlertsManagementClient:  # pylint: disable=client-accepts-api-version-keyword
    """AlertsManagement Client.

    :ivar alert_processing_rules: AlertProcessingRulesOperations operations
    :vartype alert_processing_rules:
     azure.mgmt.alertsmanagement.operations.AlertProcessingRulesOperations
    :ivar prometheus_rule_groups: PrometheusRuleGroupsOperations operations
    :vartype prometheus_rule_groups:
     azure.mgmt.alertsmanagement.operations.PrometheusRuleGroupsOperations
    :ivar operations: Operations operations
    :vartype operations: azure.mgmt.alertsmanagement.operations.Operations
    :ivar alerts: AlertsOperations operations
    :vartype alerts: azure.mgmt.alertsmanagement.operations.AlertsOperations
    :ivar smart_groups: SmartGroupsOperations operations
    :vartype smart_groups: azure.mgmt.alertsmanagement.operations.SmartGroupsOperations
    :param credential: Credential needed for the client to connect to Azure. Required.
    :type credential: ~azure.core.credentials.TokenCredential
    :param subscription_id: The ID of the target subscription. Required.
    :type subscription_id: str
    :param base_url: Service URL. Default value is "https://management.azure.com".
    :type base_url: str
    """

    def __init__(
        self,
        credential: "TokenCredential",
        subscription_id: str,
        base_url: str = "https://management.azure.com",
        **kwargs: Any
    ) -> None:
        self._config = AlertsManagementClientConfiguration(
            credential=credential, subscription_id=subscription_id, **kwargs
        )
        self._client = ARMPipelineClient(base_url=base_url, config=self._config, **kwargs)

        client_models = {k: v for k, v in _models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)
        self._serialize.client_side_validation = False
        self.alert_processing_rules = AlertProcessingRulesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.prometheus_rule_groups = PrometheusRuleGroupsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.operations = Operations(self._client, self._config, self._serialize, self._deserialize)
        self.alerts = AlertsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.smart_groups = SmartGroupsOperations(self._client, self._config, self._serialize, self._deserialize)

    def _send_request(self, request: HttpRequest, **kwargs: Any) -> HttpResponse:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = client._send_request(request)
        <HttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.HttpResponse
        """

        request_copy = deepcopy(request)
        request_copy.url = self._client.format_url(request_copy.url)
        return self._client.send_request(request_copy, **kwargs)

    def close(self) -> None:
        self._client.close()

    def __enter__(self) -> "AlertsManagementClient":
        self._client.__enter__()
        return self

    def __exit__(self, *exc_details) -> None:
        self._client.__exit__(*exc_details)
